package com.heaton.forum;
import javax.servlet.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;
import java.util.*;
import java.io.*;
import java.sql.*;

/**
 * Example program from Chapter 11
 * JSTL: JSP Standard Template Library
 * Published by Sams Publishing
 *
 *
 * This file holds the database session. This
 * object is stored as a session variable.
 *
 * @author Jeff Heaton(http://www.jeffheaton.com)
 * @version 1.0
 */

public class Session {

/**
 * A constant that contains the name of the FORUM
 * session attribute.
 */
  public final static String FORUM = "forum";

/**
 * The database driver used by this session.
 */
  private String driver;

/**
 * The database url used by this session.
 */
  private String url;


/**
 * The constructor. Constructs a Session object
 * from a page context. To create a session
 * object you should use the getSession factory
 * method.
 *
 * @param driver A database driver
 * @param driver A database url
 */
  Session(String driver,String url)
  {
    this.url = url;
    this.driver = driver;
  }

/**
 * Factory method to create sessions. This method
 * will ensure that there is only one object created
 * per user session.
 *
 * @return A Connection object.
 * @exception JspException Thrown on general error.
 */
  Connection getConnection()
  throws JspException
  {
    try {
        Class.forName(driver);
        Connection conn = DriverManager.getConnection(url);
        return conn;
    } catch ( ClassNotFoundException e ) {
      throw new JspException(e);
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
  }


/**
 * Factory method to create sessions. This method
 * will ensure that there is only one object created
 * per user session.
 *
 * @param ctx The page context.
 * @return A Session object.
 * @exception JspException Thrown on general error.
 */
  static Session getSession(PageContext ctx)
  throws JspException
  {
      Session session = (Session)ctx.getAttribute(FORUM,PageContext.SESSION_SCOPE);
      if ( session==null ) {

        String driver = ctx.getServletContext().getInitParameter("database-driver");
        String url = ctx.getServletContext().getInitParameter("database-url");
        session = new Session(driver,url);
        ctx.setAttribute(FORUM,session,PageContext.SESSION_SCOPE);
	  }
      return session;
  }


/**
 * Called to create a new user.
 *
 * @param id The new user's id.
 * @param pw The new user's password.
 * @return A user object.
 * @exception JspException Thrown on general error.
 */
  public User createUser(String id,String pw)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      stmt = connection.prepareStatement
                               ("insert into t_users(c_uid,c_pwd,c_accesses,c_first,c_last,c_bad,c_posted,c_type) "+
                                "values(?,?,?,?,?,?,?,?)");
      stmt.setString(1,id.toLowerCase());
      stmt.setString(2,pw);
      stmt.setInt(3,1);
      java.sql.Date now = new java.sql.Date(new java.util.Date().getTime());
      stmt.setDate(4,now);
      stmt.setDate(5,now);
      stmt.setInt(6,0);
      stmt.setInt(7,0);
      stmt.setString(8,"G");
      stmt.execute();

      User user = loadUser(id);
      return user;
    } catch ( SQLException e ) {
      throw new JspException(e);
    }

    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }

/**
 * Called to load a user from the database.
 *
 * @param id The user that is to be loaded.
 * @return A user object.
 * @exception JspException Thrown on general error.
 */
  public User loadUser(String id)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {

	  connection = getConnection();
      stmt = connection.prepareStatement
                               ("select c_uid,c_pwd,c_accesses,c_first,c_last,c_bad,c_posted,c_type " +
                                "from t_users " +
                                "where c_uid=?");
      stmt.setString(1, id.toLowerCase());
      ResultSet rs = stmt.executeQuery();
      if ( !rs.next() )
        return null;

      User user = new User();
      user.setId(rs.getString("c_uid"));
      user.setPassword(rs.getString("c_pwd"));
      user.setAccesses(rs.getInt("c_accesses"));
      user.setFirst(rs.getDate("c_first"));
      user.setLast(rs.getDate("c_last"));
      user.setBad(rs.getInt("c_bad"));
      user.setPosted(rs.getInt("c_posted"));
      user.setType(rs.getString("c_type"));
      return user;
    } catch ( SQLException e ) {
      throw new JspException(e);
    }

    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}

  }


/**
 * Called to return a collection of all
 * of the users.
 *
 * @return A collection that contains all of
 * the users.
 * @exception JspException Thrown on general error.
 */
  public Collection listUsers()
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      Collection c = new LinkedList();
      stmt = connection.prepareStatement
                               ("select c_uid,c_pwd,c_accesses,c_first,c_last,c_bad,c_posted,c_type " +
                                "from t_users order by c_uid" );
      ResultSet rs = stmt.executeQuery();
      while ( rs.next() ) {
        User user = new User();
        user.setId(rs.getString("c_uid"));
        user.setPassword(rs.getString("c_pwd"));
        user.setAccesses(rs.getInt("c_accesses"));
        user.setFirst(rs.getDate("c_first"));
        user.setLast(rs.getDate("c_last"));
        user.setBad(rs.getInt("c_bad"));
        user.setPosted(rs.getInt("c_posted"));
        user.setType(rs.getString("c_type"));
        c.add(user);
      }
      return c;
    } catch ( SQLException e ) {
      throw new JspException(e);
    }

    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}

  }


/**
 * Called to save the specified
 * user to the database.
 *
 * @param user The user that is to be saved.
 * @exception JspException Thrown on general error.
 */
  public void saveUser(User user)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {

	  connection = getConnection();
      stmt = connection.prepareStatement
                               ("update t_users set c_pwd = ?,c_accesses = ?,c_first = ?,c_last = ?,c_bad = ?,c_posted = ?,c_type = ? " +
                                " where c_uid=?");
      stmt.setString(1,user.getPassword());
      stmt.setInt(2,user.getAccesses());
      stmt.setDate(3,new java.sql.Date(user.getFirst().getTime()));
      stmt.setDate(4,new java.sql.Date(user.getLast().getTime()));
      stmt.setInt(5,user.getBad());
      stmt.setInt(6,user.getPosted());
      stmt.setString(7,user.getType());
      stmt.setString(8,user.getId());
      stmt.execute();
    } catch ( SQLException e ) {
      throw new JspException(e);
    }

    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }

/**
 * Called to list all of the forums
 * to a collection.
 *
 * @return A list of all forums.
 * @exception JspException Thrown on general error.
 */
  public Collection listForums()
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      Collection c = new LinkedList();
      stmt = connection.prepareStatement
                               ("select c_code,c_name,c_sequence from t_forums order by c_sequence");
      ResultSet rs = stmt.executeQuery();
      while ( rs.next() ) {
        Forum forum = new Forum();
        forum.setCode(rs.getString("c_code"));
        forum.setName(rs.getString("c_name"));
        forum.setSequence(rs.getInt("c_sequence"));
        c.add(forum);
      }
      return c;
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }


/**
 * Called to load the specified forum.
 *
 * @param id The code of the forum that is to be loaded.
 * @return The forum that was loaded.
 * @exception JspException Thrown on general error.
 */
  public Forum loadForum(String id)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();

      stmt = connection.prepareStatement
                               ("select c_code,c_name,c_sequence " +
                                "from t_forums " +
                                "where c_code=?");
      stmt.setString(1, id.toUpperCase());
      ResultSet rs = stmt.executeQuery();
      if ( !rs.next() )
        return null;

      Forum forum = new Forum();
      forum.setCode(rs.getString("c_code"));
      forum.setName(rs.getString("c_name"));
      forum.setSequence(rs.getInt("c_sequence"));
      return forum;
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}

  }


/**
 * Called to save the specified forum.
 *
 * @param forum The forum that is to be saved.
 * @exception JspException Thrown on general error.
 */
  public void saveForum(Forum forum)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {

	  connection = getConnection();
      stmt = connection.prepareStatement
                               ("update t_forums set c_name = ?,c_sequence = ? " +
                                " where c_code=?");
      stmt.setString(1,forum.getName());
      stmt.setInt(2,forum.getSequence());
      stmt.setString(3,forum.getCode());
      stmt.execute();
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }

/**
 * Called to create a new forum.
 *
 * @param code The code for the new forum.
 * @param name The name of the new forum.
 * @param sequence The sequence number of the forum. This is used to determine
 * the order that forums will be displayed in.
 * @exception JspException
 */
  public void createForum(String code,String name,int sequence)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      code = code.toUpperCase();
      stmt = connection.prepareStatement
                               ("insert into t_forums(c_code,c_name,c_sequence) "+
                                "values(?,?,?)");
      stmt.setString(1, code.toUpperCase());
      stmt.setString(2, name);
      stmt.setInt(3, sequence);
      stmt.execute();
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}

  }

/**
 * Called to delete a forum.
 *
 * @param code The code of the forum that is to be deleted.
 * @exception JspException Thrown on general error.
 */
  public void deleteForum(String code)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      code = code.toUpperCase();
      stmt = connection.prepareStatement
                               ("delete from t_forums where c_code = ?");
      stmt.setString(1, code.toUpperCase());
      stmt.execute();
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }

/**
 * Called to delete a user.
 *
 * @param id The id of the user that is to be deleted.
 * @exception JspException Thrown on general error.
 */
  public void deleteUser(String id)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      id = id.toLowerCase();
      stmt = connection.prepareStatement
                               ("delete from t_users where c_uid = ?");
      stmt.setString(1, id);
      stmt.execute();
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }



/**
 * Called to list the messages.
 *
 * @param code The forum that the messages are to be listed for.
 * @return Returns a collection of the forum messages.
 * @exception JspException Thrown on general error.
 */
  public Collection listMessages(String code)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      Collection c = new LinkedList();
      stmt = connection.prepareStatement
                               ("select c_number,c_posted,c_subject,c_sender,c_message from t_messages "+
                                "where c_forum_code = ? order by c_number");
      stmt.setString(1, code.toUpperCase());
      ResultSet rs = stmt.executeQuery();
      while ( rs.next() ) {
        Message msg = new Message();
        msg.setForumCode(code);
        msg.setNumber(rs.getInt("c_number"));
        msg.setPosted(rs.getDate("c_posted"));
        msg.setSubject(rs.getString("c_subject"));
        msg.setSender(rs.getString("c_sender"));
        msg.setMessage(rs.getString("c_message"));
        c.add(msg);
      }
      return c;
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}

  }


/**
 * Called to create a new forum message.
 *
 * @param code The forum that the message is to be created in.
 * @param from The login id of the user that this message is from.
 * @param subject The subject of this message.
 * @param message The actual message text.
 * @exception JspException Thrown on general error.
 */
  public void createMessage(String code,String from,String subject,String message)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      code = code.toUpperCase();
      stmt = connection.prepareStatement
                               ("select max(c_number)+1 as msgnum from t_messages"+
                                " where c_forum_code = ?");
      stmt.setString(1, code.toUpperCase());
      ResultSet rs = stmt.executeQuery();
      int num;
      if ( rs.next() )
        num = rs.getInt("msgnum");
      else
        num = 1;

      stmt = connection.prepareStatement
             ("insert into t_messages(c_forum_code,c_number,c_posted,c_subject,c_sender,c_message)"+
              "values(?,?,now(),?,?,?)");
      stmt.setString(1,code);
      stmt.setInt(2,num);
      stmt.setString(3,subject);
      stmt.setString(4,from);
      stmt.setString(5,message);
      stmt.execute();

      User user = loadUser(from);
      user.setPosted(user.getPosted()+1);
      saveUser(user);
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }

/**
 * Called to delete a forum message.
 *
 * @param code The forum that contains the message to be deleted.
 * @param number The number of the message to be deleted.
 * @exception JspException Thrown on general error.
 */
  public void deleteMessage(String code,int number)
  throws JspException
  {
	Connection connection = null;
	PreparedStatement stmt = null;

    try {
	  connection = getConnection();
      code = code.toUpperCase();
      stmt = connection.prepareStatement
                               ("delete from t_messages "+
                                " where c_forum_code = ? and c_number = ?");
      stmt.setString(1, code.toUpperCase());
      stmt.setInt(2, number);
      stmt.execute();
    } catch ( SQLException e ) {
      throw new JspException(e);
    }
    finally
    {
		try
		{
		if(stmt!=null)
			stmt.close();
		if(connection!=null)
			connection.close();
		}
        catch ( SQLException e ) {
          throw new JspException(e);
        }
	}
  }


/**
 * Called to get the int constant for a character scope
 * value such as "page.
 *
 * @param str A string that should contain "page", "session", etc.
 * @return The numeric form of the specified scope.
 */
  static public int getScope(String str)
  {
    if ( str==null )
      return PageContext.PAGE_SCOPE;

    if ( str.equalsIgnoreCase("request") )
      return PageContext.REQUEST_SCOPE;
    else if ( str.equalsIgnoreCase("session") )
      return PageContext.SESSION_SCOPE;
    else if ( str.equalsIgnoreCase("application") )
      return PageContext.APPLICATION_SCOPE;
    else return PageContext.PAGE_SCOPE;

  }

}
